#include <QtGui>
#include <QKeyEvent>
#include <QEvent>
#include <QFrame>
#include <string>
#include <cstdlib>
#include <sstream>
#include <fstream>
#include <list>
#include <iostream>

#include "window.h"

window::window(QWidget *parent)
  : QMainWindow(parent)
{
  mTimePoint = -1;
  mFocalPlane = -1;

  mainLabel = new QLabel(this);
  mainLabel->setEnabled(true);
  setCentralWidget(mainLabel);

  imageLabel = new QLabel(mainLabel);
  imageLabel->setEnabled(false);
  imageLabel->setGeometry(QRect(5, 5, 300, 300));

  dsName = new QLabel(mainLabel);
  //  dsName->setFont(QFont("Times", 18, QFont::Bold));
  dsName->setEnabled(false);
  dsName->setGeometry(QRect(5, 310, 290, 25));

  timePoint = new QLCDNumber(mainLabel);
  timePoint->setSegmentStyle(QLCDNumber::Flat);
  timePoint->setEnabled(false);
  timePoint->setGeometry(QRect(5, 340, 48, 25));

  focalPlane = new QLCDNumber(mainLabel);
  focalPlane->setSegmentStyle(QLCDNumber::Flat);
  focalPlane->setEnabled(false);
  focalPlane->setGeometry(QRect(5, 370, 48, 25));

  scrollTime = new QScrollBar(mainLabel);
  scrollTime->setEnabled(false);
  scrollTime->setGeometry(QRect(60, 344, 245, 16));
  scrollTime->setOrientation(Qt::Horizontal);

  scrollFocal = new QScrollBar(mainLabel);
  scrollFocal->setEnabled(false);
  scrollFocal->setGeometry(QRect(60, 374, 245, 16));
  scrollFocal->setOrientation(Qt::Horizontal);

  connect(scrollTime, SIGNAL(valueChanged(int)), this, SLOT(imageTimeChanged(int)));
  connect(scrollFocal, SIGNAL(valueChanged(int)), this, SLOT(imageFocalChanged(int)));
  connect(scrollTime, SIGNAL(valueChanged(int)), timePoint, SLOT(display(int)));
  connect(scrollFocal, SIGNAL(valueChanged(int)), focalPlane, SLOT(display(int)));

  createActions();
  createMenus();
  /*
  QIcon icon;
  icon.addPixmap(QPixmap(QString::fromUtf8("4dimageview.png")), QIcon::Normal, QIcon::Off);
  setWindowIcon(icon);
  */
  setWindowTitle(tr("4D Image Viewer"));
  resize(310, 425);
  show();
}

void window::keyPressEvent(QKeyEvent *e)
{
  if (mTimePoint != -1 && mFocalPlane != -1) {
    switch(e->key()) {
    case Qt::Key_H:
    case Qt::Key_Left:
      prevTime();
      break;
    case Qt::Key_L:
    case Qt::Key_Right:
      nextTime();
      break;
    case Qt::Key_K:
    case Qt::Key_Up:
      upFocal();
      break;
    case Qt::Key_J:
    case Qt::Key_Down:
      downFocal();
      break;
    default:
      break;
    }
  }
}

void window::wheelEvent(QWheelEvent *e)
{
  if (mTimePoint != -1 && mFocalPlane != -1) {
    if (e->delta() > 0)
      upFocal();
    else 
      downFocal();
  }
}

void window::paintEvent(QPaintEvent *ip)
{
  if (mTimePoint != -1 && mFocalPlane != -1) {
    if (pTime != mTimePoint || pFocal != mFocalPlane) {
      QString imageName = path + "/img/" + dataset + "_t" + QString::fromStdString(digit3(mTimePoint)) + "_z" + QString::fromStdString(digit3(mFocalPlane)) + "_ch000.png";
      imageName = QDir::toNativeSeparators(imageName);

      std::cerr << imageName.toStdString() << std::endl;

      pTime = mTimePoint;
      pFocal = mFocalPlane;

      QImage image(imageName);
      QPixmap piximg = QPixmap::fromImage(image);
      QPainter paint(&piximg);

      imageLabel->setPixmap(piximg);
      update();
    }
  }
}

void window::open()
{
  QString fileName = QFileDialog::getOpenFileName(this, tr("OpenFile"), QDir::currentPath(), tr("IV files (*.iv)"));

  if (!fileName.isEmpty()) {
    QFileInfo info(fileName);
    path = info.absolutePath();

    std::string filestd = fileName.toStdString();
    std::ifstream file;
    file.open(filestd.c_str());
    std::string tag;
    std::string name;
    int val;
    while (file >> tag) {
      if (!strcmp(tag.c_str(), "dataset")) {
	file >> name;
	dataset = QString::fromStdString(name);
	std::cerr << name << std::endl;
      } else if (!strcmp(tag.c_str(), "width")) {
	file >> val;
	width = val;
	std::cerr << width << std::endl;
      } else if (!strcmp(tag.c_str(), "height")) {
	file >> val;
	height = val;
	std::cerr << height << std::endl;
      } else if (!strcmp(tag.c_str(), "mint")) {
	file >> val;
	minTime = val;
	std::cerr << minTime << std::endl;
      } else if (!strcmp(tag.c_str(), "maxt")) {
	file >> val;
	maxTime = val;
	std::cerr << maxTime << std::endl;
      } else if (!strcmp(tag.c_str(), "minz")) {
	file >> val;
	minFocal = val;
	std::cerr << minFocal << std::endl;
      } else if (!strcmp(tag.c_str(), "maxz")) {
	file >> val;
	maxFocal = val;
	std::cerr << maxFocal << std::endl;
      }
    }

      //    int loc = filestd.find(".iv", 0);
      //    filestd.erase(loc);
      //    dataset = QString::fromStdString(filestd);

    imageLabel->setGeometry(5, 5, width, height);
    imageLabel->setEnabled(true);

    QPointer<QLabel> ds = dsName;
    ds->setText(dataset);
    dsName->setGeometry(QRect(5, height + 10, width - 10, 25));
    dsName->setEnabled(true);

    timePoint->setGeometry(QRect(5, height + 40, 48, 25));
    timePoint->setEnabled(true);

    focalPlane->setGeometry(QRect(5, height + 70, 48, 25));
    focalPlane->setEnabled(true);

    scrollTime->setGeometry(QRect(60, height + 44, width - 63, 16));
    scrollTime->setEnabled(true);

    scrollFocal->setGeometry(QRect(60, height + 74, width - 63, 16));
    scrollFocal->setEnabled(true);

    scrollTime->setRange(minTime, maxTime);
    scrollFocal->setRange(minFocal, maxFocal);
    mTimePoint = minTime;
    mFocalPlane = (maxFocal + minFocal) / 2;
    scrollTime->setValue(mTimePoint);
    scrollFocal->setValue(mFocalPlane);

    resize(width + 10, height + 125);
  } else {
    return;
  }
}

std::string window::digit3(int a)
{
  std::stringstream str;
  str.width(3);
  str.fill('0');
  str << a;
  std::string ret;
  str >> ret;
  return ret;
}

void window::nextTime()
{
  if (mTimePoint < maxTime) {
    mTimePoint++;
    scrollTime->setValue(mTimePoint);
  }
}

void window::prevTime()
{
  if (mTimePoint > minTime) {
    mTimePoint--;
    scrollTime->setValue(mTimePoint);
  }
}

void window::upFocal()
{
  if (mFocalPlane > minFocal) {
    mFocalPlane--;
    scrollFocal->setValue(mFocalPlane);
  }
}

void window::downFocal()
{
  if (mFocalPlane < maxFocal) {
    mFocalPlane++;
    scrollFocal->setValue(mFocalPlane);
  }
}

void window::imageFocalChanged(int a)
{
  mFocalPlane = a;
  scrollFocal->setValue(mFocalPlane);
}

void window::imageTimeChanged(int a)
{
  mTimePoint = a;
  scrollTime->setValue(mTimePoint);
}

void window::about()
{
  QMessageBox::about(this, tr("About 4D Image Viewer"), tr("4D Image Viewer\n2009 RIKEN ASI"));
}

void window::createActions()
{
  openAct = new QAction(tr("&Open..."), this);
  openAct->setShortcut(tr("Ctrl+O"));
  connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

  exitAct = new QAction(tr("E&xit"), this);
  exitAct->setShortcut(tr("Ctrl+Q"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  aboutAct = new QAction(tr("&About"), this);
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

  aboutQtAct = new QAction(tr("About &Qt"), this);
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void window::createMenus()
{
  fileMenu = new QMenu(tr("&File"), this);
  fileMenu->addAction(openAct);
  fileMenu->addSeparator();
  fileMenu->addAction(exitAct);

  helpMenu = new QMenu(tr("&Help"), this);
  helpMenu->addAction(aboutAct);
  helpMenu->addAction(aboutQtAct);

  menuBar()->addMenu(fileMenu);
  menuBar()->addMenu(helpMenu);
}
