#ifndef WINDOW_H
#define WINDOW_H

#include <QMainWindow>
#include <QFrame>
#include <QtGui>
#include <string>
#include <list>

QT_BEGIN_NAMESPACE
class QAction;
class QLabel;
class QPainter;
class QMenu;
class QScrollArea;
class QScrollBar;
QT_END_NAMESPACE

class window : public QMainWindow
{
  Q_OBJECT

public:
  window(QWidget* parent = 0);

private slots:
  void open();

  void about();
  void keyPressEvent(QKeyEvent *);
  void prevTime();
  void nextTime();
  void upFocal();
  void downFocal();

  void imageTimeChanged(int);
  void imageFocalChanged(int);

protected:
  void wheelEvent(QWheelEvent *);
  void paintEvent(QPaintEvent *);

private:
  void createActions();
  void createMenus();

  std::string digit3(int);

  QLabel *imageLabel;
  QScrollArea *scrollArea;
  QLabel *mainLabel;
  QLabel *dsName;

  QLCDNumber *timePoint;
  QLCDNumber *focalPlane;
  QScrollBar *scrollTime;
  QScrollBar *scrollFocal;

  QString dataset;
  QString path;
  int mTimePoint;
  int mFocalPlane;
  int minTime;
  int maxTime;
  int minFocal;
  int maxFocal;

  int pTime;
  int pFocal;

  int width;
  int height;

  QAction *openAct;
  QAction *exitAct;
  QAction *aboutAct;
  QAction *aboutQtAct;

  QMenu *fileMenu;
  QMenu *helpMenu;
};

#endif
